function [dpos,varargout] = pseudoinv01(cur_pos,dtarget,forward_fcn,fparams,iparams,varargin)
% PSEUDOINV01 A pseudoinverter function
%   This function uses a local approximant of the pseudoinverse (inverse Jacobian)
%   to determine the desired change in position required to attain the desired change
%   in output.
%
%   <Inputs>
%   cur_pos     : position vector or columnwise matrix (CM) in the same space as the
%                 the input to the forward_model
%   dtarget     : desired change in target vector or CM in the same space as the 
%                 output of the forward model
%   forward_fcn : m-file implementing the forward model
%   fparams     : parameters for the forward function
%   iparams     : parameters for this function
%   varargin    : other miscellaneous parameters
%
%   <Outputs>
%   dpos        : desired change in position corresponding to desired change in
%                 target
%   varargout   : misc outputs

% Original version written by Alfonso Nieto-Castanon

% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /tools/uiwaitbar.m 5     4/09/02 3:45p Satra $

% Store parameters in local variables
if nargin<5 | isempty(iparams),
    iparams.DELTA = 0.5;
    iparams.ALPHA = 0.5;
    iparams.BETA  = 0.05;
    iparams.LAMBDA= 0;
    iparams
end;

DELTA = iparams.DELTA;
ALPHA = iparams.ALPHA;
BETA  = iparams.BETA;
LAMBDA= iparams.LAMBDA;

% Determine the number of inputs and outputs
Ninput   = size(cur_pos,2);
Ntarget = size(dtarget,2);

% Determine number of elemenets in input and output space
dimpos  = size(cur_pos,1);
dimtarget = size(dtarget,1);
cur_target  = feval(forward_fcn,cur_pos,fparams);

for ninput = 1:Ninput,
    % Current target
    cur_target  = feval(forward_fcn,cur_pos(:,ninput),fparams);
    % Determine the search grid
    gridpos     = cur_pos(:,ninput)*ones(1,dimpos)+DELTA*diag(ones(1,dimpos));
    gridtarget  = cur_target(:,ones(1,dimpos));
    
    % Determine the Jacobian
    J = (feval(forward_fcn,max(min(gridpos,4),-4),fparams)-gridtarget)./DELTA;
    
    % Determine the inverse of the Jacobian
    %   Various tricks can be played here related to regularization
    %   This step is crucial for the inverse transformation and various
    %   different techniques can be applied to create a better solution
    %   At present the straightforward pseudoinverse is applied.
    J(find(isnan(J(:)))) = 0;
    [U,S,V]=svd(J,0);
    %Jinv=V*pinv(S)*U';
    Jinv = pinv(J'*J+LAMBDA*eye(size(J,2)))*J';
    %Jinv=J';
    
    % Determine the null space of the forward transformation
    % This will fail if the target dimension is greater than position dimension
    % To avoid this for the time being using try/catch
    %try
        Jnull=V(:,dimtarget+1:dimpos); 
        Jnull=Jnull*pinv(Jnull'*Jnull)*Jnull';
        %catch
        %Jnull = 0;
        %end
    
    for ntarget = 1:Ntarget,
        % Jacobian formula
        dpos(:,ntarget) = ALPHA*Jinv*dtarget(:,ntarget) - BETA*Jnull*cur_pos(:,ninput);
        %[dtarget,-(cur_target+dtarget)+feval(forward_fcn,cur_pos(:,ninput)+0.5*dpos(:,ntarget),fparams)]
    end;
end;